//Sample using LiquidCrystal library
#include <LiquidCrystal.h>

/*******************************************************

This program will test the LCD panel and the buttons
Mark Bramwell, July 2010

********************************************************/

// select the pins used on the LCD panel
LiquidCrystal lcd(8, 9, 4, 5, 6, 7);

// define some values used by the panel and buttons
int lcd_key     = 0;
int adc_key_in  = 0;
volatile byte is_alarm_set = 0;
volatile byte hour = 11;
volatile byte minutes = 30;
volatile byte seconds = 0;
volatile byte hour_alarm = 0;
volatile byte minutes_alarm = 0;
volatile byte seconds_alarm = 0;

volatile byte choose_alarm = 0;
#define btnRIGHT  0
#define btnUP     1
#define btnDOWN   2
#define btnLEFT   3
#define btnSELECT 4
#define btnNONE   5

float last_temp = 0.f;
// read the buttons
ISR(TIMER1_COMPA_vect) {
  seconds++;

  if (seconds > 59) {
    minutes++;
    seconds = 0;
    if (minutes > 59) {
       hour++; 
       minutes = 0;
       if (hour > 23) {
          hour = 0;
       }
      }
    }
}

void print_time(byte h, byte m, byte s) {
  lcd.setCursor(6,1);
  if (h < 10) {
      lcd.print("0");
    }
    lcd.print(h);
    lcd.print(":");
    if(m < 10) {
      lcd.print("0");
    }
    lcd.print(m);
    print_temp();
}

void set_alarm() {
  delay(500);
  lcd.setCursor(0,0);
  lcd.print("P SEL WHEN RDY");
  byte min_aux = minutes;
  byte sec_aux = seconds;
  while(read_LCD_buttons() != btnSELECT) {
    if (read_LCD_buttons() == btnUP) {
      increase_hour_alarm();
    } else if (read_LCD_buttons() == btnDOWN) {
      decrease_hour_alarm();
    } else if (read_LCD_buttons() == btnLEFT) {
      decrease_minutes_alarm();
    } else if (read_LCD_buttons() == btnRIGHT) {
      increase_minutes_alarm();
    }
    delay(200);
    print_time(hour_alarm, minutes_alarm, seconds_alarm);
  }
  is_alarm_set = 1;
//  Serial.println(hour_alarm);
//  Serial.println(minutes_alarm);
//  Serial.println();
//  Serial.println(hour);
//  Serial.println(minutes);
}
void increase_hour() {
  noInterrupts();
  hour++;
  if (hour >= 24) {
    hour = 0;
  }
  interrupts();
}
void decrease_hour() {  
  noInterrupts();
  hour--;
  if (hour == 255) {
    hour = 23;
  }
  interrupts();
}
void increase_minutes() {
  noInterrupts();
  minutes++;
  if (minutes >= 60) {
    minutes = 0;
  }
  interrupts();
}
void decrease_minutes() {
  noInterrupts();
  minutes--;
  if (minutes == 255) {
    minutes = 59;
  }
  interrupts();
}

void increase_hour_alarm() {
  noInterrupts();
  hour_alarm++;
  if (hour_alarm >= 24) {
    hour_alarm = 0;
  }
  interrupts();
}
void decrease_hour_alarm() {  
  noInterrupts();
  hour_alarm--;
   if (hour_alarm < 0) {
    hour_alarm = 23;
  }
  interrupts();
}
void increase_minutes_alarm() {
  noInterrupts();
  minutes_alarm++;
  if (minutes_alarm >= 60) {
    minutes_alarm = 0;
  }
  interrupts();
}
void decrease_minutes_alarm() {
  noInterrupts();
  minutes_alarm--;
   if (minutes_alarm < 0) {
    minutes_alarm = 59;
  }
  interrupts();
}

byte Bell[8] = {
0b00100,
0b01110,
0b01110,
0b01110,
0b11111,
0b00000,
0b00100,
0b00000
};

byte Circle[8] = {
0b00000,
0b00100,
0b01010,
0b00100,
0b00000,
0b00000,
0b00000,
0b00000
};

int read_LCD_buttons()
{
 adc_key_in = analogRead(0);      
 if (adc_key_in > 1000) return btnNONE; 
 if (adc_key_in < 50)   return btnRIGHT;
 if (adc_key_in < 250)  return btnUP;
 if (adc_key_in < 400)  return btnDOWN;
 if (adc_key_in < 420 )  return btnLEFT;
 if (adc_key_in < 850 && adc_key_in > 410)  return btnSELECT;
 
 return btnNONE;
}

void setup()
{
  TCCR1A = 0X00;
  TCCR1B = (1<<CS10) | (1<<CS12) | (1<<WGM12);
  OCR1A = 15625;
  TIMSK1 = (1<<OCIE1A);

 lcd.createChar(0, Bell);
 lcd.createChar(1, Circle);
 Serial.begin(9600);
 lcd.begin(16, 2);              // start the library
 lcd.setCursor(0,0);
 pinMode(A2, OUTPUT);
 digitalWrite(A2, HIGH);
}

void print_temp() {
  int val = analogRead(A1);  // read the input pin
  

  float voltage = (val * 5.0)/1024.0f;
  float temp = (voltage - 0.5) * 100;
  if (abs(temp - last_temp) > 1) {
    last_temp = temp;
  }
//  Serial.print("Temp : ");
//  Serial.println(temp);
  lcd.setCursor(12, 1);
  lcd.print(int(last_temp));
  lcd.write(byte(1));
  lcd.print("C");
}

void loop()
{
 lcd.clear();
 lcd.setCursor(0,0);
 lcd.print("PUSH SEL FOR ");
 lcd.write(byte(0));
 // move to the begining of the second line
 lcd_key = read_LCD_buttons();  // read the buttons
 if (hour == hour_alarm && minutes == minutes_alarm && is_alarm_set) {
 digitalWrite (A2, LOW);
 delay (5000);
 digitalWrite (A2, HIGH);
 delay (500);
 is_alarm_set = 0;

 }
 switch (lcd_key)               // depending on which button was pushed, we perform an action
 {
   case btnRIGHT:
     {
     increase_minutes();
     delay(100);
     break;
     }
   case btnLEFT:
     {
     decrease_minutes();
     delay(100);
     break;
     }
   case btnUP:
     {
     increase_hour();
       delay(100);
     break;
     }
   case btnDOWN:
     {
     decrease_hour();
       delay(100);
     break;
     }
   case btnSELECT:
     {
     if (!is_alarm_set) {
      set_alarm();
     }
     break;
     }
     case btnNONE:
     {
//     lcd.write(byte(0));
     break;
     }
 }

 print_time(hour, minutes, seconds);
 delay(100);
// print_temp();

  
}
